#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <stdlib.h>
#include <portbits.h>
#include "global.h"
#include "lcd.h"
#include "uart.h"

//***************************************************************************
//*																			*
//*						LCD Controller fr 320x240 LCD						*
//*																			*
//*								mit UART									*
//*																			*
//*								Version 1.00								*
//*																			*
//*								 by Benedikt								*
//*																			*
//*						Email:	benedikt83 t gmx.net						*
//*																			*
//***************************************************************************
//*																			*
//*	Die Software darf frei kopiert und verndert werden, solange sie nicht	*
//*	ohne meine Erlaubnis fr kommerzielle Zwecke eingesetzt wird.			*
//*																			*
//***************************************************************************

#define XTAL_CPU			16000000UL
#define UART_BAUD_RATE		   19200UL

int main(void)
{	PORTD=3;
	DDRB=255;
	DDRD=254;
	DDRE=255;
	PORTA=255;
	PORTC=255;

	lcd_init();						// LCD Controller initialisieren
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE, XTAL_CPU));

	unsigned char c, ypos=0;
	unsigned short xpos=0;
	unsigned char textcol=255, backcol=0;

	for (;;)
	{	c = uart_getchar();
		if (c>31)					// Display Text
		{	lcd_writechar(xpos/8,ypos,c,textcol,backcol);
			xpos+=TEXT_X;
			if (xpos>=XSIZE)
			{	xpos=0;
				ypos+=TEXT_Y;
				if (ypos>=YSIZE)
					ypos=0;
			}
		}
		else if (c==1)				// Cursor Home
		{	xpos=0;
			ypos=0;
		}
		else if (c==2)				// LCD off
		{	ENABLE_VLCD=0;
		}
		else if (c==3)				// LCD on
		{	ENABLE_VLCD=1;
		}
		else if (c==8)				// Backspace
		{	xpos-=TEXT_X;
			if (xpos>=32768)
			{	xpos=XSIZE-TEXT_X;
				ypos-=TEXT_Y;
				if (ypos>=240)
					ypos=YSIZE-TEXT_Y;
			}
			lcd_writechar(xpos/8,ypos,32,textcol,backcol);
		}
		else if (c==9)				// Set Pixel
		{	CONVERTW x;
			unsigned char y,c;
			x.b[0]=uart_getchar();
			x.b[1]=uart_getchar();
			y=uart_getchar();
			c=uart_getchar();
			lcd_setpixel(x.w,y,c);
		}
		else if (c==10)        		// Line Feed
		{	ypos+=TEXT_Y;
			if (ypos>=YSIZE) 
				ypos=0;
		}
		else if (c==11)      	 	// Delete in place
		{	lcd_writechar(xpos/8,ypos,32,textcol,backcol);
		}
		else if (c==12)       		// Form Feed
		{	xpos=0;
			ypos=0;
			lcd_clear();
		}
		else if (c==13)				// Carriage Return
		{	xpos=0;
		}
		else if (c==14)				// Set Brightness
		{	c=uart_getchar();
		}
		else if (c==15)				// Set Contrast
		{	c=uart_getchar();
			if (c>100)
				c=100;
			OCR1B=c*2+c/2;
		}
		else if (c==16)				// Load BMP
		{	unsigned char x1,y,xs,x,c,y1,ys,mode;
			if (uart_getchar()==0xAA)
			{	x1=uart_getchar();
    			y1=uart_getchar();
    			xs=uart_getchar();
    			ys=uart_getchar();
				mode=uart_getchar();
    			for (y=0; y<ys; y++)
    			{	for (x=0; x<xs; x++)
    				{	c=uart_getchar();
						if (mode)
	    					lcd_writebyteg(x+x1,y+y1,c);
						else
	    					lcd_writebyte(x+x1,y+y1,c);
    				}
				}
			}
		}
		else if (c==17)    	    	// Set Cursor Position (Column and Row)
		{	xpos=uart_getchar()*TEXT_X;
			ypos=uart_getchar()*TEXT_Y;
			if (xpos>=XSIZE)
				xpos=0;
			if (ypos>=YSIZE)
				ypos=0;
		}
		else if (c==18)				// Draw Line
		{	CONVERTW x1,x2;
			unsigned char y1,y2;
			x1.b[0]=uart_getchar();
			x1.b[1]=uart_getchar();
			y1=uart_getchar();
			x2.b[0]=uart_getchar();
			x2.b[1]=uart_getchar();
			y2=uart_getchar();
			c=uart_getchar();
			lcd_line (x1.w,y1,x2.w,y2,c);
		}
		else if (c==19)				// Draw Circle
		{	CONVERTW x1;
			unsigned char y1,c2,r;
			x1.b[0]=uart_getchar();
			x1.b[1]=uart_getchar();
			y1=uart_getchar();
			r=uart_getchar();
			c=uart_getchar();
			c2=uart_getchar();
			lcd_circle (x1.w,y1,r,c,c2);
		}
		else if (c==21)				// Textcolor
		{	textcol=uart_getchar();
			backcol=uart_getchar();
		}
		else if (c==22)				// Enable Scroll
		{	uart_getchar();
			uart_getchar();
			uart_getchar();
		}
		else if (c==25)				// Get CGRAM
		{	c=uart_getchar();
			c&=7;
			for (unsigned char i=0; i<8; i++)
				cgram[c][i]=uart_getchar();
		}
		else if (c==28)				// Large Number
		{	uart_getchar();
			uart_getchar();
			uart_getchar();
		}
		else if (c==29)				// Draw Block
		{	CONVERTW x1,x2;
			unsigned char y1,y2,c2;
			x1.b[0]=uart_getchar();
			x1.b[1]=uart_getchar();
			y1=uart_getchar();
			x2.b[0]=uart_getchar();
			x2.b[1]=uart_getchar();
			y2=uart_getchar();
			c=uart_getchar();
			c2=uart_getchar();
			lcd_block (x1.w,y1,x2.w,y2,c,c2);
		}
		else if (c==30)				// Send directly
		{	c=uart_getchar();
			if (c==0)
				lcd_writecgram(xpos/8,ypos,uart_getchar(),textcol,backcol);
			else
				lcd_writechar(xpos/8,ypos,uart_getchar(),textcol,backcol);
   			xpos+=TEXT_X;
   			if (xpos>=XSIZE)
   			{	xpos=0;
   				ypos+=TEXT_Y;
   				if (ypos>=YSIZE)
   					ypos=0;
			}
		}
	}
}


